unit ufrmMain;

interface

uses
  Windows
  , Messages
  , SysUtils
  , Classes
  , Graphics
  , Controls
  , Forms
  , Dialogs
  , StdCtrls
  , ExtCtrls
  , BOM
  ;

type
  TfrmMain = class(TForm)
    lstUsers: TListBox;
    lstUserTypes: TListBox;
    lstUserActions: TListBox;
    Label1: TLabel;
    Label2: TLabel;
    Label3: TLabel;
    Panel1: TPanel;
    Button1: TButton;
    Button2: TButton;
    Button3: TButton;
    Label4: TLabel;
    cmbUserTypes: TComboBox;
    cmbUserActions: TComboBox;
    Label5: TLabel;
    Label6: TLabel;
    Button4: TButton;
    Button5: TButton;
    procedure Button4Click(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure Button1Click(Sender: TObject);
    procedure Button2Click(Sender: TObject);
    procedure Button3Click(Sender: TObject);
    procedure Button5Click(Sender: TObject);
    procedure lstUsersClick(Sender: TObject);
  private
    { Private declarations }
    function GetUsersByType(UserType: TUserType): TStringList;
    function GetUsersByAction(UserAction: TUserAction): TStringList;
    procedure DeleteAction(Action: TUserAction);
    procedure LoadUsers;
    procedure LoadUserTypes;
    procedure LoadUserActions;
    procedure LoadTypes;
    procedure LoadActions;
  public
    { Public declarations }
  end;

var
  frmMain: TfrmMain;

implementation

uses
  uRelMan
  ;

{$R *.DFM}

procedure TfrmMain.Button4Click(Sender: TObject);
begin
  Close;
end;

procedure TfrmMain.FormCreate(Sender: TObject);
var
  n: integer;
begin
  // Load the list of users
  LoadUsers;
  // Load the list of user types
  LoadTypes;
  // Load the list of user actions
  LoadActions;
end;

procedure TfrmMain.Button1Click(Sender: TObject);
var
  utUserType: TUserType;
  slTemp: TStringList;
begin
  // List all the users whose UserType is the same as the one selected in the
  // cmbUserTypes combo box
  if (cmbUserTypes.ItemIndex > -1) then
  begin
    lstUsers.Clear;
    utUserType := TUserType(cmbUserTypes.Items.Objects[cmbUserTypes.ItemIndex]);
    slTemp := nil;
    try { ... finally }
      slTemp := GetUsersByType(utUserType);
      lstUsers.Items.AddStrings(slTemp);
    finally
      slTemp.Free;
    end;  { try ... finally }

  end;
end;

procedure TfrmMain.Button2Click(Sender: TObject);
var
  uaUserAction: TUserAction;
  slTemp: TStringList;
begin
  // List all the users whose UserType contains the action selected in the
  // cmbUserActions combo box
  if (cmbUserActions.ItemIndex > -1) then
  begin
    lstUsers.Clear;
    uaUserAction := TUserAction(cmbUserActions.Items.Objects[cmbUserActions.ItemIndex]);
    slTemp := nil;
    try { ... finally }
      slTemp := GetUsersByAction(uaUserAction);
      lstUsers.Items.AddStrings(slTemp);
    finally
      slTemp.Free;
    end;  { try ... finally }

  end;
end;

function TfrmMain.GetUsersByAction(UserAction: TUserAction): TStringList;
var
  n, m: integer;
  lstTypes, lstUsers: TList;
begin
  // Return a string list containing the user names of all the users whose
  // user type's actions contain the supplied object
  Result := TStringList.Create;
  lstTypes := nil;
  try { ... finally }
    // Get a list of usertypes containing this useraction
    lstTypes := gRelMan.FindObjects(nil, UserAction, riTypeAction);
    // For each of the usertypes, get a list of the users with that usertype
    for n := 0 to Pred(lstTypes.Count) do
    begin
      if (TObject(lstTypes.Items[n]) is TUserType) then
      begin
        lstUsers := nil;
        try { ... finally }
          // Get the list of users for the current usertype
          lstUsers := gRelMan.FindObjects(nil, lstTypes.Items[n], riUserType);
          // Add the usernames to the string list
          for m := 0 to Pred(lstUsers.Count) do
          begin
            if (TObject(lstUsers.Items[m]) is TUser) then
            begin
              Result.AddObject(TUser(lstUsers.Items[m]).UserName, lstUsers.Items[m]);
            end;
          end;
        finally
          lstUsers.Free;
        end;  { try ... finally }

      end;
    end;
  finally
    lstTypes.Free;
  end;  { try ... finally }

end;

function TfrmMain.GetUsersByType(UserType: TUserType): TStringList;
var
  n: integer;
  lstTemp: TList;
begin
  // Return a string list containing the user names of all the users whose
  // user type matches the supplied object
  Result := TStringList.Create;
  lstTemp := nil;
  try { ... finally }
    // Get a list of the user objects with this usertype
    lstTemp := gRelMan.FindObjects(nil, UserType, riUserType);
    // Add the usernames to the string list
    for n := 0 to Pred(lstTemp.Count) do
    begin
      if (TObject(lstTemp.Items[n]) is TUser) then
      begin
        Result.AddObject(TUser(lstTemp.Items[n]).UserName, lstTemp.Items[n]);
      end;
    end;
  finally
    lstTemp.Free;
  end;  { try ... finally }

end;

procedure TfrmMain.Button3Click(Sender: TObject);
var
  n: integer;
begin
  // List all the users
  LoadUsers;
end;

procedure TfrmMain.LoadUserActions;
var
  n: integer;
  usrTemp: TUser;
  lstTemp: TList;
begin
  // Load the list of actions for the currently selected user
  lstUserActions.Clear;
  usrTemp := TUser(lstUsers.Items.Objects[lstUsers.ItemIndex]);
  lstTemp := nil;
  try { ... finally }
    lstTemp := gRelMan.FindObjects(usrTemp.UserType, nil, riTypeAction);
    for n := 0 to Pred(lstTemp.Count) do
    begin
      if (TObject(lstTemp.Items[n]) is TUserAction) then
      begin
        lstUserActions.Items.Add(TUserAction(lstTemp.Items[n]).Name);
      end;
    end;
  finally

  end;  { try ... finally }

end;

procedure TfrmMain.LoadUsers;
var
  n: integer;
begin
  // Load the list of users
  lstUsers.Clear;
  for n := 0 to Pred(UserList.Count) do
  begin
    lstUsers.Items.AddObject(TUser(UserList.Items[n]).UserName, UserList.Items[n]);
  end;
end;

procedure TfrmMain.LoadUserTypes;
var
  n: integer;
begin
  // Display the user type for the currently selected user
  lstUserTypes.Clear;
  lstUserTypes.Items.Add(TUser(lstUsers.Items.Objects[lstUsers.ItemIndex]).UserType.Name);
end;

procedure TfrmMain.Button5Click(Sender: TObject);
var
  uaTemp: TUserAction;
begin
  // Delete a user action
  if (cmbUserActions.ItemIndex > -1) then
  begin
    uaTemp := TUserAction(cmbUserActions.Items.Objects[cmbUserActions.ItemIndex]);
    DeleteAction(uaTemp);
    LoadUserActions;
    LoadActions;
  end;
end;

procedure TfrmMain.DeleteAction(Action: TUserAction);
var
  {n, }i: integer;
//  lstTemp: TList;
begin
  // First, remove any references to the action in the UserTypes' lists of
  // actions
{  lstTemp := nil;
  try} { ... finally }
{    // Get a list of usertypes that contain this action
    lstTemp := gRelMan.FindObjects(nil, Action, riTypeAction);
    // Now remove the relationships
    for n := 0 to Pred(lstTemp.Count) do
    begin
      gRelMan.RemRelationship(lstTemp.Items[n], Action, riTypeAction);
    end;
  finally
    lstTemp.Free;
  end;}  { try ... finally }
  gRelMan.DelObject(Action);

  // Delete the user action from the list of user actions. As it is a TObjectList
  // it will also free the UserAction object.
  i := UserActionList.IndexOf(Action);
  if (i > -1) then
  begin
    UserActionList.Delete(i);
  end;
end;

procedure TfrmMain.LoadActions;
var
  n: integer;
begin
  // Load all the actions into the combo box
  cmbUserActions.Clear;
  for n := 0 to Pred(UserActionList.Count) do
  begin
    cmbUserActions.Items.AddObject(TUserAction(UserActionList.Items[n]).Name, UserActionList.Items[n]);
  end;
end;

procedure TfrmMain.LoadTypes;
var
  n: integer;
begin
  // Load all the types into the combo box
  cmbUserTypes.Clear;
  for n := 0 to Pred(UserList.Count) do
  begin
    cmbUserTypes.Items.AddObject(TUserType(UserTypeList.Items[n]).Name, UserTypeList.Items[n]);
  end;
end;

procedure TfrmMain.lstUsersClick(Sender: TObject);
begin
  LoadUserTypes;
  LoadUserActions;
end;

end.
